"""
Constants used throughout KiCad Library Manager.
"""

# Metadata filenames
GITHUB_METADATA_FILE = "kilm.yaml"
CLOUD_METADATA_FILE = ".kilm_metadata"

# Configuration paths and directories
CONFIG_DIR_NAME = "kicad-lib-manager"
CONFIG_FILE_NAME = "config.yaml"

# Template-related constants
TEMPLATES_DIR = "templates"
TEMPLATE_METADATA = "metadata.yaml"
TEMPLATE_CONTENT_DIR = "template"
HOOKS_DIR = "hooks"

# Default configuration values
DEFAULT_MAX_BACKUPS = 5
DEFAULT_LIBRARIES: list = []
